#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2018 Neko404NotFound
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in a$
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""
Abstract base classes for the pag module.
"""


__all__ = ("PagABC",)


from abc import ABC, abstractmethod
import weakref


class PagABC(ABC):
    """
    Keeps track of the references. Useful for memory usage debugging:
    a serious downside to the pagination system in Neko2.

    The weak references are dealt with automatically internally, so you
    never even have to acknowledge it's existence.
    """

    _instances = weakref.WeakSet()

    def __init__(self):
        self._instances.add(self)

    @classmethod
    @abstractmethod
    def memory_usage(cls) -> int:
        """Estimates the memory usage used by the internal content."""
        ...
